/*!
 * SphereSlider - slider
 * @version		1.0.0
 * @uses MooTools 1.3
 * @author Constantin Boiangiu <info [at] constantinb.com>
 * Copyright Youjoomla.com
 */
var SphereSlider = new Class({
    Implements: [Options, Events],
    options: {
        outsidecont: null,
        container: null,
        items: null,
        images: null,
        autoSlide: 0,
        zoomingStep: .2, // percent between full image and zoomed out image
        visible: 2, // thumbnails visible on previous next
        visiblePortion: .2, // percent from the image visible when in background
        imageWidth: 565,
        imageHeight: 377,
		backOpacity:0.5,
        backZoom: .1, // zoom percent for images in background
        onInit: function (currentIndex, elements) {

            var getNavHeight = $(this.options.outsidecont).getElement('.spherenavs').getSize().y / 2;
            var setPosition = ($(this.options.container).getSize().y / 2) - getNavHeight;
            $(this.options.outsidecont).getElements('.spherenavs').setStyle('top', setPosition);
			
			
//			this.elements.each(function (el, i) {
//				//console.log(i);
//				var getWidth = el.getSize().x;
//				var getHeight= el.getSize().y;
//				el.getElement('img').setStyles({'width':getWidth,'height':getHeight});
//			});	

        },
        onChange: function (prevIndex, currentIndex, elements) {
            // to get the current element
            var currentElement = elements[currentIndex];
            var outcont = $(this.options.outsidecont);
            var getTitle = currentElement.getElement('.ititle').get('html');
            var getDesc = currentElement.getElement('.desc').get('html');

            currentElement.getElement(this.options.images).removeClass('spherehidden').addClass('sphereselected');
            outcont.getElement('.clonedesc').morph({
                'opacity': 0
            });

            (function () {
                outcont.getElement('.title').set({
                    'html': getTitle
                });
                outcont.getElement('.intro').set({
                    'html': getDesc
                });
                outcont.getElement('.clonedesc').morph({
                    'opacity': 1
                })
            }).delay(500);

            var prevElement = elements[prevIndex];
            prevElement.getElement(this.options.images).removeClass('sphereselected').addClass('spherehidden');
			

        },
        _debug: false
    },
    initialize: function (options) {
        this.setOptions(options);

        this.hasConsole = typeof console !== 'undefined';
        // check if items are defined
        if (!this.options.items) {
            this.logError('No items specified. Script dropped.');
            return;
        }
        // get the list elements
        this.elements = $(this.options.container).getElements(this.options.items);
        this.images = $(this.options.container).getElements(this.options.images);
        // check the number of elements. must be at least this.visible*2+1
        var totalVisible = this.options.visible * 2 + 1;
        if (totalVisible > this.elements.length) {
            this.logError('Number of items is smaller than displayed items. You want to display ' + totalVisible + ' but have only ' + this.elements.length + ' items in slider. Script dropped.');
            return;
        }
        // check if image width and height are set
        if (!this.options.imageWidth || !this.options.imageHeight) {
            this.logError('No image width and/or height specified in script parameters. Script dropped.');
            return;
        }

        this.current = 0;
        this.start();
        this.resize();

        this.fireEvent('onInit', [this.current, this.elements]);

        this.autoRotation = null;
        this.isPaused = false;
        this.autoRotate();
        this.runNav();

    },


    runNav: function () {

        var self = this;

        $(this.options.outsidecont).getElement('.prev').addEvent('click', function (e) {
            e.preventDefault();
            self.prev();
        });
        $(this.options.outsidecont).getElement('.next').addEvent('click', function (e) {
            e.preventDefault();
            self.next();
        });
    },


    // starts auto rotation
    autoRotate: function () {
        var self = this;

        if (this.autoRotation || this.isPaused) return;
        if (this.options.autoSlide && this.options.autoSlide > 0) {
            this.autoRotation = this.gotoNext.periodical(this.options.autoSlide, this);
        }
    },
    // stops auto rotation
    stopRotate: function () {

        var self = this;
        if (this.autoRotation) {
            clearInterval(this.autoRotation);
            this.autoRotation = null;
        }
    },

    pauseRotate: function () {
        this.isPaused = true;
        this.stopRotate();
    },

    next: function () {
        var self = this;
        this.stopRotate();
        this.gotoNext();
        this.autoRotate();
    },

    prev: function () {
        var self = this;
        this.stopRotate();
        this.gotoPrev();
        this.autoRotate();
    },

    // goes to next element
    gotoNext: function () {
        var ind = this.current + 1 >= this.elements.length ? 0 : this.current + 1;
        this.rotate(ind);
    },
    // goes to previous element
    gotoPrev: function () {
        var ind = this.current - 1 < 0 ? this.elements.length - 1 : this.current - 1;
        this.rotate(ind);
    },

    // passes messages to console if it exists and debug is on
    logError: function (message) {
        if (this.options._debug && this.hasConsole) {
            console.log('Internal script error: ' + message);
        }
    },

    resize: function (prevIndex, currentIndex, elements) {
        var self = this;

        var outcont = $(this.options.outsidecont);
        var getTitle = outcont.getElement(self.options.images).getElement('.ititle').get('html');
        var getDesc = outcont.getElement(self.options.images).getElement('.desc').get('html');
		var Opaci	= this.options.backOpacity;

        window.onresize = function () {
            self.start();
            outcont.getElements(self.options.images).removeClass('sphereselected');
            outcont.getElement(self.options.images).removeClass('spherehidden').addClass('sphereselected');
			outcont.getElements('img').morph({opacity:Opaci});
			outcont.getElement('img').morph({opacity:1});
            outcont.getElement('.title').set({
                'html': getTitle
            });
            outcont.getElement('.intro').set({
                'html': getDesc
            });

        }



    },
    start: function () {


        var self = this;


        // size all elements to the image size passed in options
        this.elements.setStyles({
            'width': this.options.imageWidth,
            'height': this.options.imageHeight
        });
        this.images.setStyles({
            'width': this.options.imageWidth,
            'height': this.options.imageHeight
        });
		
        var containerSize = $(this.options.container).getSize();

        $(this.options.container).addEvents({
            'mouseenter': this.pauseRotate.bind(this),
            'mouseleave': function () {
                this.isPaused = false;
                this.autoRotate();
            }.bind(this)
        });

        this.positions = {};
        this.positions.main = {
            'zIndex': 100,
            'width': this.options.imageWidth,
            'height': this.options.imageHeight,
            'left': (containerSize.x - this.options.imageWidth) / 2,
            'top': (containerSize.y - this.options.imageHeight) / 2
        };
        this.positions.back = {
            'zIndex': 50,
            'width': this.options.imageWidth * this.options.backZoom,
            'height': this.options.imageHeight * this.options.backZoom,
            'left': (containerSize.x - this.options.imageWidth * this.options.backZoom) / 2,
            'top': (containerSize.y - this.options.imageHeight * this.options.backZoom) / 2
        };
        this.positions.right = {};
        this.positions.left = {};

        for (var i = 1; i <= this.options.visible; i++) {
            var zIndex = 100 - i,
                imgWidth = this.options.imageWidth - (this.options.imageWidth * this.options.zoomingStep * i),
                imgHeight = this.options.imageHeight - (this.options.imageHeight * this.options.zoomingStep * i);

            var mainElem = this.positions.main;

            // calculate the right positioned elements
            var posX = mainElem.width + mainElem.left - (imgWidth - imgWidth * this.options.visiblePortion),
                posY = (containerSize.y - imgHeight) / 2;

            Object.each(this.positions.right, function (o, i) {
                posX += o.width * this.options.visiblePortion;
            }.bind(this))

            this.positions.right[i] = {
                'zIndex': zIndex,
                'width': imgWidth,
                'height': imgHeight,
                'left': posX,
                'top': posY
            };

            // calculate the left positioned elements
            var posX = mainElem.left - imgWidth * this.options.visiblePortion;
            Object.each(this.positions.left, function (o, i) {
                posX -= o.width * this.options.visiblePortion;
            }.bind(this))

            this.positions.left[i] = {
                'zIndex': zIndex,
                'width': imgWidth,
                'height': imgHeight,
                'left': posX,
                'top': posY
            };
        }

        this.elements.setStyles(this.positions.back);
        this.images.setStyles(this.positions.back);
		this.images.getElement('img').morph({opacity:this.options.backOpacity});

        // position the main centered element
        this.elements[this.current].setStyles(this.positions.main);
        this.images[this.current].setStyles(this.positions.main);
		//this.images[this.current].getElement('img').setStyles(this.positions.main);
		this.images[this.current].getElement('img').morph({opacity:1});
		




        // position elements on the right
        for (var i = 1; i <= this.options.visible; i++) {
            this.elements[i].setStyles(this.positions.right[i]);
            this.images[i].setStyles(this.positions.right[i]);
			//this.images[i].getElement('img').setStyles(this.positions.right[i]);
        }
        // position elements on the left
        var l = this.elements.length - 1,
            i = 1;
        for (var t = l; t > l - this.options.visible; t--) {
            this.elements[t].setStyles(this.positions.left[i]);
            this.images[t].setStyles(this.positions.left[i]);
			//this.images[t].getElement('img').setStyles(this.positions.left[i]);
            i++;
        }

        this.elements.each(function (el, i) {
            el.set('morph', {
                'duration': 600,
                'link': 'cancel'
            });
            this.images[i].set('morph', {
                'duration': 600,
                'link': 'cancel'
            });

            var imgParent = this.images[i].getParent('a');
            if (imgParent) {
                imgParent.addEvent('click', function (e) {
                    if (i !== this.current) {
                        e.preventDefault();
                    }
                }.bind(this));
            }

            el.addEvent('click', function (event) {
                this.stopRotate();
                this.rotate(i);
                this.autoRotate();
            }.bind(this));
        }.bind(this));
    },

    rotate: function (i) {
        var self = this;

        if (i == this.current) return;
        // get the sequences of elements for current and next
        var currentSeq = this.getSequence(this.current);
        nextSeq = this.getSequence(i);
        // position the current element in center	
        this.elements[i].setStyle('z-index', 101).morph(this.positions.main);
        this.images[i].setStyle('z-index', 101).morph(this.positions.main);
		this.images[i].getElement('img').morph({opacity:1});

        this.fireEvent('onChange', [this.current, i, this.elements]);

        // hide all elements not present in next sequence
        currentSeq.all.each(function (el, i) {
            if (nextSeq.all.indexOf(el) == -1) {
                this.elements[el].morph(this.positions.back);
                this.images[el].morph(this.positions.back);
            }
        }.bind(this));
        // do the morph for right side
        Object.each(nextSeq.left, function (e, i) {
            this.elements[e].morph(this.positions.left[i]);
            this.images[e].morph(this.positions.left[i]);
			this.images[e].getElement('img').morph({opacity:this.options.backOpacity});
        }.bind(this));
        // do the morph for left side
        Object.each(nextSeq.right, function (e, i) {
            this.elements[e].morph(this.positions.right[i]);
            this.images[e].morph(this.positions.right[i]);
			this.images[e].getElement('img').morph({opacity:this.options.backOpacity});
        }.bind(this));
        // set the current element
        this.current = i;
    },

    getSequence: function (index) {

        var t = this.elements.length,
            v = this.options.visible,
            indexes = {
                'main': index,
                'left': {},
                'right': {},
                'all': []
            };

        indexes.all.include(index);

        for (var i = 1; i <= v; i++) {
            // right elements		
            var rightInd = index + i >= t ? index + i - t : index + i;
            indexes.right[i] = rightInd;
            indexes.all.include(rightInd);
            // left elements	
            var leftInd = index - i < 0 ? t - (-1 * (index - i)) : index - i;
            indexes.left[i] = leftInd;
            indexes.all.include(leftInd);
        }

        return indexes;
    }
})